%% Main script for fitting procedure

%--------------------------------------------------------------------------
% This script includes an example of RESET switching using TEAM model with
% opposite polarity (team_op_model, op for opposite). Since we are
% interested in RESET switching the procedure tries to optimize three
% parameters responsible for the RESET switching, i_off, k_off, and a_off.
% This script can be modified to perform the fitting procedure for I-V
% curve, ON switching, OFF switching, TEAM, VTEAM, and different
% polarities. Follow the comments!
%--------------------------------------------------------------------------

close all;
clear all;
clc;

%--------------------------------------------------------------------------
% input_parameters = vector includes the parameters you wish to optimize
% for example --> input_parameters = [i_off,k_off,a_off]; if you wish to 
% perform fitting to a RESET switching using TEAM model.

% Write initial values ofr the parameters.
% NOTE: sometimes the algorithms converges to a local minimum, which is
% far from a good result! so try to change the initial parameters in this
% case
input_parameters = %init values --> [i_off_init,k_off_init,a_off_init];

%Define the lower and upper bound of the parameters
lb  = %lower bounds [i_off_lb,k_off_lb,a_off_lb];
ub  = %upper bounds [i_off_ub,k_off_ub,a_off_ub];
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Here, two optimizationa algorithms are used. The first takes into account
% lower and upper bound, while the other do not!
% The results (optimized parameters) are returned to optres1 and optres2.
% MSE function is implemented in a seperate file which include the fitting
% criterion.
optres1=simulannealbnd(@mse,input_parameters,lb,ub);
optres2=fminunc(@mse,input_parameters);
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
% This section will help you perform visual interpretation of the
% optimization results.

v_ref   = % Insert the voltage vector of the reference data. 
i_ref   = % Insert the current vectore of the reference data.
t_ref   = % Insert the time steps vector of the reference data.

% Define the parameters of the model
D       =  % Device length 
x_init  =  % init value for state variable [0,D]
iv      =  % I-V relationship, linear = 0, exponential=1.

% Define your input
v_input     = v_ref; 
t_vec       = t_ref;

% Run the desired model in this case the team_op_model is used.
 [v_model1,i_model1,~]=team_op_model(v_input,t_vec,x_init,iv,optres1);
 [v_model2,i_model2,~]=team_op_model(v_input,t_vec,x_init,iv,optres2);
 
%--------------------------------------------------------------------------
 
%--------------------------------------------------------------------------
% Draw the results
figure(1);
 
% Plot the results of the first optimization algorithm (in red) vs.
% reference data (in blue)
subplot(2,1,1);
hold on;
% In this case I want to plot R vs. t.
% If you wnat I-V curve change to plot(v,i).
plot(t_vec,v_ref./i_ref,'r');
plot(t_vec,v_model1./i_model1);
title('Simulated Annealing');

% Plot the results of the second optimization algorithm (in red) vs.
% reference data (in blue)
subplot(2,1,2);
hold on;
% In this case I want to plot R vs. t.
% If you wnat I-V curve change to plot(v,i).
plot(t_vec,v_ref./i_ref,'r');
plot(t_vec,v_model2./i_model2);
title('Gradient Descent');
%--------------------------------------------------------------------------


